<?

/***************************************************************************
 *   Copyright (C) 2005 by Ferenc Veres   *
 *   lion@netngine.hu   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
* Recreates the Xaraya locale files from the database
*
* Does exactly the opposite than TranslationImportXaraya. Note, that dead
* projects won't be exported.
*/

class TranslationExportXaraya implements TranslationExport
{

	private $RootTimeStamp;	/**< Timestamp of the root project, older projects won't be exported as they are consdered to be dead projects */
	private $SilentMode; /**< Supress all messages while exporting? **/

	/**
	* Export the translation strings of one project and return as updated document xml string.
	*
	* Gets the passed project's strings from the database and puts them in the
	* project's $Skeleton document, returns the resulting string.
	* @param $project The TranslationProject to created exported XML file for.
	* @returns string The resulting translated XML document.
	*/
	
	private function &ExportProject(TranslationProject $project)
	{
		$langXml = new DOMDocument();
		//$langXml->substituteEntities = true;
		$langXml->LoadXML($project->GetSkeleton());

		// Loop on all strings of this project and put in a hash for later replace
		$strings = new StringIterator($project->GetID());
		foreach($strings as $str)
		{
			$projStrings["###".$str->GetStringID()."###"] = $this->FixEntity($str->GetOriginal());
			
			$projStrings["###".$str->GetStringID()."t###"] = $this->FixEntity($str->GetTranslation());
/*			if(preg_match('/\&/', $str->GetOriginal()))
			{
				echo( $str->GetOriginal()."\n");
				echo(FixQuot($str->GetOriginal()."\n"));
			}
*/		}

		// XPath to all translation entries
		$xpath = new DOMXPath($langXml);
		$xpath->registerNamespace("m","http://xaraya.com/2002/ns/translations");
		$entryNodes = $xpath->query("//m:translations/m:entry");

		// Loop on each string in theXML and replace variables
		foreach ($entryNodes as $entry)
		{
			$origNodes = $xpath->query("m:string", $entry);
			$transNodes = $xpath->query("m:translation", $entry);

			if($origNodes->item(0)->nodeValue == "")
			{
				echo "Empty Original node in the file:".$project->GetName().".\n";
			}
			else
			{
				// Replace ### node with the same strings hash key value
				$origNodes->item(0)->nodeValue = $projStrings[$origNodes->item(0)->nodeValue];
				
				$transNodes->item(0)->nodeValue = $projStrings[$transNodes->item(0)->nodeValue];

			}
		}
		$xmlContent =& $langXml->saveXML();
		return $xmlContent;
	}

	/**
	* Fix string for using in XML output. This may need changes with other DOMDocument versions.
	*
	* Becuase the current domdocument actions look broken, thus we need to fix some things here.
	* @param $text The text to escape.
	* @returns string: The escaped text &,<,>," replaced with entities.
	*/

	function FixEntity($text)
	{
		$s = str_replace("&", "&amp;", $text);
		$s = str_replace("\"", "&quot;", $s); // !!!
		$s = str_replace("<", "&lt;", $s);
		$s = str_replace(">", "&gt;", $s);
		return $s;
	}

	/**
	* Create directories and translated files according to the project tree and strings database.
	*
	* Calls ->ExportProject() on all translation projects recurisvely from the given root.
	* This is a recursive function which calls itself for subdirectories.
	* @param $dir String, the directory where the subdirectories and files should be created.
	* @param $projectIterator The iterator initialized with the root project from where we export.
	*/
	
	private function ProcessRecursive($dir, ProjectIterator $projectIterator)
	{
		// Process each project in the iterator
		foreach($projectIterator as $project)
		{
			// Completely ignore dead projects and their subprojects (older than root)
			if($this->RootTimeStamp <= $project->GetLastSeen())
			{
				// Empty document means direcrory only.
				if($project->GetDocument() == "")
				{
					// Create a new dir and process the connected projects recursively.
	
					$newDir = $dir."/".$project->GetName();
					if(!is_dir($newDir))
					{
						$ret = mkdir($newDir);
						if(!$ret)
						{
							throw new Exception("Failed to create directory $newDir, no permission or disk full.");
						}
					}
					if($projectIterator->hasChildren())
					{
						$this->ProcessRecursive($newDir, $projectIterator->getChildren());
					}
				}
				else
				{
					// Save one project XML file.
					
					$export = $this->ExportProject($project);
					$bytes = file_put_contents($dir."/".$project->GetName(), $export);
					if($bytes != strlen($export))
					{
						throw new Exception("Failed to save  ".$dir."/".$project->GetName().", no permission or disk full.");
					}
					if(!$this->SilentMode)
					{
						echo("Saved: ".$dir."/".$project->GetName()."\n");
					}
				}
			}
		}

	}
	
	/**
	* Export finished Xaraya translations of the selected locale into the selected directory.
	*
	* Exporting will process all projects in the given locale's Xaraya root, create
	* a language pack directory strucure on the $CONFIG configured destination.
	* @param $localeName string Name of the locale to be exported (e.g. "hu_HU.utf-8").
	* @param $silentMode bool true value suppresses all output messages.
	*/
	
	public function Export($localeName, $silentMode = false)
	{
		$this->SilentMode = $silentMode;
		
		$localeID = TranslationProject::$LocaleIDCache[$localeName];
		if(empty($localeID))
		{
			throw new Exception("ExportXaraya called with unknown locale name: $localeName");
		}

		// Check export path and create locale dir
		if(!is_dir(GetConfigVar('export_path')))
		{
			throw new Exception("Export path points to non-existing directory: ".GetConfigVar('export_path'));
		}
		$dir = GetConfigVar('export_path')."/".$localeName;
		if(!is_dir($dir))
		{
			$ret = mkdir($dir, 0755);
			if(!$ret)
			{
				throw new Exception("Failed to create directory $dir, no permission or disk full.");
			}
		}
		$dir = GetConfigVar('export_path')."/".$localeName."/xml/";
		if(!is_dir($dir))
		{
			$ret = mkdir($dir, 0755);
			if(!$ret)
			{
				throw new Exception("Failed to create directory $dir, no permission or disk full.");
			}
		}
		
		// Go exporting
		$rootProject = new TranslationProject();
		$rootProject->LoadRoot("Xaraya", $localeID);
		$this->RootTimeStamp = $rootProject->GetLastSeen();
		
		$allProjects = new ProjectIterator($rootProject->GetID(), $localeID);

		$this->ProcessRecursive($dir, $allProjects);
	}
}
?>
