<?
/***************************************************************************
 *   Copyright (C) 2005 by Ferenc Veres   *
 *   lion@netngine.hu   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
 * This class can handle projects. A project is either a file which contains
 * some translatable strings, or an empty project which does not have actual
 * strings, just subprojects only.
 *
 * A project(file) has an original document, a skeleton which contains ###33###
 * type references to TranslationString objects. The translated document can be
 * created from the Project's Skeleton and the Strings referenced by the ## IDs.
 */
 
class TranslationProject
{
	private $ProjectID;	/**< Project's ID if loaded */
	private $ParentID;	/**< Project's parent project id */
	private $Name;		/**< Project name */
	private $LocaleID;	/**< Project locale ID */
	private $Skeleton;	/**< Project's original file with placeholders for the translatable strings */
	private $Document;	/**< The original untranslated, unchanged file of this project */
	private $LastSeen;	/**< The date and time (UNIX timestamp) when this project was last updated. */
	
	public static $LocaleIDCache; /**< Static hash table caching the locale names(key) and IDs(value) */
	
	/**
	* Initialize a new project handler.
	*
	* @param $loadByID Integer, a project ID to be loaded. If not specified, empty project handler is made.
	*/
	
	public function __construct($loadByID = 0)
	{
		if($loadByID != 0)
		{
			$this->LoadByID($loadByID);
		}
	}

	/**
	* Initialize the static locale ID cache, by loading locales from the database.
	*
	* You can access the locale IDs in TraslationProject::$LocaleIDCache[localename] = id format.
	*/
	
	public static function InitLocaleIDCache()
	{
		$DB =& GetDbConn();
		$Tables =& GetTables();
		
		$rs = $DB->Execute("SELECT ID, Name FROM ".$Tables["Locales"]." ORDER BY Name");
		while($row = $rs->FetchRow()) 
		{
			TranslationProject::$LocaleIDCache[$row["Name"]] = $row["ID"];
		}
	}
	
	private function Clear()
	{
		unset($this->ProjectID);
		unset($this->ParentID);
		unset($this->Name);
		unset($this->LocaleID);
		unset($this->Skeleton);
		unset($this->Document);
		unset($this->lastSeen);
	}

	private function NameCollision()
	{
		$DB =& GetDbConn();
		$Tables =& GetTables();

		$old = $DB->GetOne("SELECT ID FROM ".$Tables["Projects"]." WHERE ParentID = ? AND Name = ? AND LocaleID = ?", array($this->ParentID, $this->Name, $this->LocaleID));
		if($old)
		{
			return true;
		}
		else
		{
			return false;
		}

	}

	/**
	* Create a new project into the current object instance, which has 0 as parent ID.
	*
	* Root projects cannot have actual strings content, just a placeholder for
	* all its subprojects
	*
	* @param $name Name of the project to be created.
	* @param $localeID Locale ID of this project.
	* @returns The ID of the newly created project, false if a root project wiht $name already exists.
	*/
	
	public function CreateRootProject($name, $localeID)
	{
		return $this->CreateProject(0, $name, $localeID);
	}

	/**
	* Initialize this object as a new (sub)project in a root or any other project.
	*
	* These projects may or may not store the real textural content for the project's translation.
	* If you want to store the project in the database, you must call ->Save() too. Until saving,
	* the project won't have a Project ID!
	* 
	* @param $parentID In which project to create this project as child.
	* @param $name Name of the project to be created.
	* @param $localeID Locale ID of this project.
	* @param $document The original content of the document to be translated.
	* @param $skeleton The document which contains IDs in place of each translation string from origdoc.
	* @exception 'Exception' thrown if a project with same name exists in the same location.
	*/
	
	public function CreateProject($parentID, $name, $localeID, $document = '', $skeleton = '')
	{
		$this->Clear();
		
		$this->ParentID = $parentID;
		$this->Name = $name;
		$this->LocaleID = $localeID;
		$this->Document = $document;
		$this->Skeleton = $skeleton;
		$this->LastSeen = time();
		
		// Project with this name already exists in this Parent?
		if($this->NameCollision())
		{
			$this->Clear();
			throw new Exception("Project with this name already exists, same location.");
		}
	}

	/**
	* Save the current project in the database (insert or update).
	*
	* The project may have empty skeleton and original document, as a placeholder project.
	* But name, parentid and LocaleID are mandatory to be set.
	*
	* @returns The ID of the created project.
	*/
	
	public function Save()
	{
		$DB =& GetDbConn();
		$Tables =& GetTables();
		
		// Is this an already saved project?
		if(isset($this->ProjectID))
		{
			// Already saved (or a loaded) project, UPDATE in database
			
			$ok = $DB->Execute("UPDATE ".$Tables["Projects"]." SET ParentID = ?, Name = ?, LocaleID = ?, OrigDoc = ?, Skeleton = ?, LastSeen = now() WHERE ID = ?", array($this->ParentID, $this->Name, $this->LocaleID, $this->Document, $this->Skeleton, $this->ProjectID));
		}
		else
		{
			// New project, INSERT into database
			
			// Project with this name already exists in this Parent?
			if($this->NameCollision())
			{
				throw new Exception("Project with this name already exists, same location.");
			}
			
			$ok = $DB->Execute("INSERT INTO ".$Tables["Projects"]." (ParentID, Name, LocaleID, OrigDoc, Skeleton, LastSeen) VALUES (?,?,?,?,?, now())", array($this->ParentID, $this->Name, $this->LocaleID, $this->Document, $this->Skeleton));

			// Get new ID
			if(!$DB->Insert_ID())
			{
				throw new Exception("DB Error: Insert_ID not supported.");
			}
			$this->ProjectID = $DB->Insert_ID();
		}

		return $this->ProjectID;
	}

	/**
	* Load a root project with the given name (which has parent ID 0).
	*
	* @param $name Which root project to look for.
	* @param $localeID Locale ID of the requested project.
	* @return True on success, false otherwise.
	*/
	
	public function LoadRoot($name, $localeID)
	{
		return $this->Load(0, $name, $localeID);
	}

	/**
	* Load a project by name.
	*
	* @param $parentID In which parent project.
	* @param $name Which project to look for.
	* @param $localeID Which locale version to find.
	* @return True on success, false otherwise.
	*/
	
	public function Load($parentID, $name, $localeID)
	{
		$DB =& GetDbConn();
		$Tables =& GetTables();
		
		$rs = $DB->Execute("SELECT ID, ParentID, Name, LocaleID, Skeleton, OrigDoc, TIMESTAMP(LastSeen) as LastSeen FROM ".$Tables["Projects"]." WHERE ParentID = ? AND Name = ? AND LocaleID = ?", array($parentID, $name, $localeID));

		$retVal = $this->LoadRecordSetEntry($rs);
		$rs->Close();

		return $retVal;
	}

	/**
	* Load the next project from an existing ADODB recordset.
	*
	* The query running in the recordset must return the same columns as the
	* query in this class' ->Load() method. Useful for loading a lot of project at once.
	*
	* @param $recordSet Existing ADODB recordset running a project list query.
	* @return True on success, false otherwise.
	*/
	
	public function LoadFromRecordset($recordSet)
	{
		return $this->LoadRecordSetEntry($recordSet);
	}

	/**
	* Load a project by ID.
	*
	* @param $projectID Which project to load.
	* @return True on success, false otherwise.
	*/
	
	public function LoadByID($projectID)
	{
		$DB =& GetDbConn();
		$Tables =& GetTables();

		$rs = $DB->Execute("SELECT ID, ParentID, Name, LocaleID, Skeleton, OrigDoc, TIMESTAMP(LastSeen) as LastSeen FROM ".$Tables["Projects"]." WHERE ID = ?", array($projectID));
		
		$retVal = $this->LoadRecordSetEntry($rs);
		$rs->Close();
		
		return $retVal;

	}

	/**
	* Load a project's details using an exisitng record set.
	*
	* Called by public Load() methods.
	* @param $rs The recordset running the specified query and providing data for the project.
	* @returns True on success, false otherwise (no more rows).
	*/
	
	private function LoadRecordSetEntry($rs)
	{
		$this->Clear();
		
		if($row = $rs->FetchRow())
		{
			$this->ProjectID = $row["ID"];
			$this->ParentID = $row["ParentID"];
			$this->Name = $row["Name"];
			$this->LocaleID = $row["LocaleID"];
			$this->Skeleton = $row["Skeleton"];
			$this->Document = $row["OrigDoc"];
			$this->LastSeen = $row["LastSeen"];
			return true;
		}
		else
		{
			return false;
		}
	}

	/* SET METHODS */

	/**
	* Set the name of this project
	*
	* To apply changes, you must ->Save() into the database.
	* @param $name String New name for this project.
	*/
	public function SetName($name)
	{
		if(!$name)
		{
			throw new Exception("Cannot set Project Name to emtpy.");
		}
		$this->Name = $name;
	}
	
	/**
	* Set the skeleton of this project
	*
	* To apply changes, you must ->Save() into the database.
	* @param $skeleton String New name for this project.
	*/
	public function SetSkeleton($skeleton)
	{
		$this->Skeleton = $skeleton;
	}

	/**
	* Set the original document of this project
	*
	* To apply changes, you must ->Save() into the database.
	* @param $document New name for this project.
	*/
	public function SetDocument($document)
	{
		$this->Document = $document;
	}


	/* GET METHODS */
	
	/**
	* Get the name of this project
	*
	* @returns string Name of this project.
	*/
	public function GetName()
	{
		if(!isset($this->Name))
		{
			throw new Exception("Uninitialized project, cannot get Name.");
		}
		return $this->Name;
	}
	
	/**
	* Get the ID of this project. 0 if project is not yet in the database.
	*
	* @returns int ID of this project.
	*/
	public function GetID()
	{
		if(!isset($this->Name))
		{
			throw new Exception("Uninitialized project, cannot get ID.");
		}
		return $this->ProjectID;
	}

	/**
	* Get the Parent ID of this project.
	*
	* @returns int Parent ID of this project.
	*/
	public function GetParentID()
	{
		if(!isset($this->Name))
		{
			throw new Exception("Uninitialized project, cannot get Parent ID.");
		}
		return $this->ParentID;
	}

	/**
	* Get the ID of this project. 0 if project is not yet in the database.
	*
	* @returns int ID of this project.
	*/
	public function GetLocaleID()
	{
		if(!isset($this->Name))
		{
			throw new Exception("Uninitialized project, cannot get Locale ID.");
		}
		return $this->LocaleID;
	}

	/**
	* Get the skeleton of this project
	*
	* @returns string Skeleton of this project.
	*/
	public function GetSkeleton()
	{
		if(!isset($this->Name))
		{
			throw new Exception("Uninitialized project, cannot get Skeleton.");
		}
		return $this->Skeleton;
	}
	
	/**
	* Get the original document of this project
	*
	* @returns string Original document of this project.
	*/
	public function GetDocument()
	{
		if(!isset($this->Name))
		{
			throw new Exception("Uninitialized project, cannot get Document.");
		}
		return $this->Document;
	}

	/**
	* Get the last seen (last update) timestamp of this project
	*
	* Usually projects older than the root project of a tree can be considered as dead
	* projects (removed from the project, becuase they were not updated by the last import).
	* @returns timestamp The last update time of this project.
	*/
	public function GetLastSeen()
	{
		if(!isset($this->Name))
		{
			throw new Exception("Uninitialized project, cannot get LastSeen.");
		}
		return $this->LastSeen;
	}

}

?>
